import { Request } from "express";
import { Query } from "express-serve-static-core";

export enum USER_TYPE {
  ORGANIZATION = "ORGANIZATION",
  CANDIDATE = "CANDIDATE",
}

export type KeyDetails = {
  key: any;
  code: any;
  shiftKey: any;
  time: number;
};

export type KeystrokeType = {
  keys: string[];
  times: number[] | any;
  sums?: number[];
  means?: number[];
  sd?: number[];
  filteredMeans?: number[];
  filteredSd?: number[];
  covMatrix?: number[][];
};

export type KeystrokeDataType = {
  hold: KeystrokeType;
  flight: KeystrokeType;
  dd: KeystrokeType;
  full: KeystrokeType;
};

export type AddressType = {
  addressLine: string;
  city: string;
  country: string;
};

export type AuthType = {
  userType: USER_TYPE;
  email: string;
  userId?: string;
  password: string;
  keystrokeDataTimestamps: number[];
  keystrokeData: KeystrokeDataType;
  controls?: ControlsType;
};

export type ResumeDataType = {
  skills: string[] | null;
  degree: string[] | null;
  designation: string[] | null;
  experience: string[] | null;
  company_names: string[] | null;
  no_of_pages: number;
  total_experience: number;
};

export type CandidateType = {
  _id?: string;
  name: string;
  bio: string;
  contacts: {
    email: string;
    phone: string;
    address: AddressType;
    residentialAddress?: AddressType;
  };
  dateOfBirth: string;
  jobIds: string[];
  profilePicture: string;
  state: "INTIAL" | "READY";
  resume?: string;
  resumeData?: ResumeDataType;
};

export type OrganizationType = {
  _id?: string;
  name: string;
  description: string;
  contacts: {
    email: string;
    phone: string[];
    address: AddressType;
    website: string;
  };
  profilePicture: string;
};

export type ControlsType = {
  standard: {
    sd: number;
    threshold: number;
    use: boolean;
  };
  fullStandard: {
    threshold: number;
    use: boolean;
  };
};

export type JobType = {
  _id: string;
  title: string;
  description: string;
  primarySkills: string[];
  secondarySkills?: string[];
  salary: {
    min: number;
    max: number;
    currency: string;
  };
  applications: string[];
  organization: string;
};

export type ApplicationType = {
  candidate: string;
  job: string;
  status: "Pending" | "Accepted" | "In progress" | "Rejected";
  interview?: {
    date: string;
    time: string;
    link: string;
    videoRef?: string;
  };
  score: {
    primary: number;
    primatyMatch: string[];
    secondary: number;
    secondaryMatch: string[];
    similarity: number;
    total: number;
  };
};

export interface TypedRequest<T extends Query, U> extends Request {
  body: U;
  query: T;
}

//PAYLOADS
export type SignUpPayload = {
  passwords: string[];
  keydown: KeyDetails[][];
  keyup: KeyDetails[][];
  email: string;
  userType: USER_TYPE;
  candidate?: CandidateType;
  organization?: OrganizationType;
};

export type SignInPayload = {
  password: string;
  keydown: KeyDetails[];
  keyup: KeyDetails[];
  email: string;
  userType: USER_TYPE;
};

export type UpdatePasswordPayload = {
  passwords: string[];
  keydown: KeyDetails[][];
  keyup: KeyDetails[][];
  oldPassword: string;
};
