import { Schema, model } from "mongoose";
import { AddressType, CandidateType, ResumeDataType } from "../config/types";

const AddressSchema = new Schema<AddressType>(
  {
    addressLine: String,
    city: String,
    country: String,
  },
  { id: false }
);

const ContactsSchema = new Schema<AddressType>(
  {
    email: { type: String, require: false },
    phone: { type: String, require: false },
    address: { type: AddressSchema, require: false },
    residentialAddress: { type: AddressSchema, require: false },
  },
  { id: false }
);
const ResumeDataSchema = new Schema<ResumeDataType>(
  {
    skills: { type: [String], require: false },
    degree: { type: [String], require: false },
    designation: { type: [String], require: false },
    experience: { type: [String], require: false },
    company_names: { type: [String], require: false },
    no_of_pages: { type: Number, require: false },
    total_experience: { type: Number, require: false },
  },
  { id: false }
);

const candidateSchema = new Schema<CandidateType>({
  name: String,
  bio: String,
  contacts: { type: ContactsSchema, require: false },
  dateOfBirth: String,
  jobIds: [{ type: Schema.Types.ObjectId, ref: "jobs" }],
  profilePicture: String,
  state: { type: String, default: "INTIAL" },
  resume: { type: String, require: false },
  resumeData: { type: ResumeDataSchema, require: false },
});

const Candidates = model<CandidateType>("candidates", candidateSchema);
export default Candidates;
