
import pandas as pd
import os

from scripts.processing import document_processing
from tqdm import tqdm

import warnings
warnings.filterwarnings("ignore")

skills = {
 "primary" : ['Python', 'Machine Learning', 'node.js', 'AWS', 'Kubernetese', 'NLP', 'GCP', 'predective', 'OCR'],
 "secondary" : ['data', 'science', 'modeling', 'anomaly', 'privacy', 'visualization', 'OCR'],
}


def document_score(df):
    
    # Page score
    df.loc[df['no_of_pages'] == 1, ['page_score']] = 100
    df.loc[df['no_of_pages'] == 2, ['page_score']] = 60
    df.loc[(df['no_of_pages'] > 2) |
           (df['no_of_pages'] == 0), ['page_score']] = 30
    
    # Word score
    df.loc[(df['words'] >= 200) & (df['words'] < 400),
           ['word_score']] = 100
    df.loc[(df['words'] >= 400) & (df['words'] < 600),
           ['word_score']] = 70
    df.loc[((df['words'] > 0) & (df['words'] < 200))|
           (df['words'] > 600) | (df['words'].isnull()),
           ['word_score']] = 40
    
    df['document_score'] = (df['page_score'] + df['word_score']) * 0.25
    
    df.drop(['word_score', 'page_score'], axis=1, inplace=True)
    
    return df

if __name__=='__main__':

    resume_dir = 'resumes/'
    jd_file = 'Job_description.txt'
    list_of_resumes = os.listdir(resume_dir)  # list_of_resumes = ['Dhaval_Thakkar_Resume.pdf', 'asdasdasd.pdf']
    
    df = pd.DataFrame()
    for file in tqdm(list_of_resumes):
        result = document_processing(resume_dir+file, skills, jd_file)
        candidate = result.skills_match()
        df = pd.concat([df, candidate], ignore_index=True)
    
    df = document_score(df)
        
    # Final score
    df['Score'] = df['primary_score'] + df['secondary_score'] + df['document_score'] + df['document_similarity']
    
    df = df.sort_values('Score', ascending=False)
    df = df.reset_index(drop=True)

    print(df)
    
    df.to_csv('Candidates_score.csv', index=False)