import pandas as pd
import texthero as hero

from pyresparser.utils import extract_text

from PIL import Image
from sklearn.feature_extraction.text import CountVectorizer
from sklearn.metrics.pairwise import cosine_similarity

# skills = {
#  "primary" : ['Python', 'Machine Learning', 'node.js', 'AWS', 'Kubernetese', 'NLP', 'GCP', 'predective', 'OCR'],
#  "secondary" : ['data', 'science', 'modeling', 'anomaly', 'privacy', 'visualization', 'OCR'],
# }

class document_processing:
    
    def __init__(self, resume, skills, job_desc):
        self.resume = resume
        self.skills = skills
        self.job_desc = job_desc
        
    def extract_resume(self):
        filepath = self.resume
        
        extension = filepath.split('.')[-1]
        extension = '.'+extension

        resume_txt = extract_text(filepath, extension=extension)
        
        return resume_txt
         
    def find_unigram(df, column):
        
        unigrams  = (df[column].str.lower()
                    .str.replace(r'[^a-z\s]', '')
                    .str.split(expand=True)
                    .stack()).reset_index(drop=True)
        
        unigrams = hero.clean(unigrams)
        un_df = pd.DataFrame(unigrams, columns = ['text'])
        
        return un_df
    
    def find_match(self, source, match):
        
        # Remove the null values 
        match.dropna(inplace=True)
        match.reset_index(drop=True)
        match.columns = ['text']
        match['text'] = hero.clean(match['text'])
        
        # Find the max val
        max_val = len(match)  #9


        #1. source (main_df) Python, Python, Python, Python 
        #2. match (primary skill)       
        # Find the skills that match with the resume
        df = pd.merge(source, match, on = 'text') 
        df.drop_duplicates(inplace=True) # Python
        df.reset_index(drop=True)
        
        # Skills matching
        match_skills = len(df) #5
        
        if match_skills == 0:
            lst_skills = []
            score = 0
        elif match_skills > 0:
            lst_skills = df['text'].tolist()
            score = int((match_skills / max_val) * 100)
        
        return score, lst_skills
    
    def fill_data(self, source, target, column):
   
        source.loc[0, column] = str(target[column])
        
        return source   
    
    def resume_cosine_score(self, text):
        
        jd_txt = self.job_desc
        jd_txt = pd.Series(jd_txt)
        jd_txt = hero.clean(jd_txt)
        jd_txt = jd_txt[0]

        [[1,24],[24,1]]

        text_list = [text, jd_txt]
        cv = CountVectorizer()
        count_matrix = cv.fit_transform(text_list)
        match_percentage = cosine_similarity(count_matrix)[0][1] * 100
        match_percentage = round(match_percentage, 2)

        return match_percentage
        
    
    def skills_match(self):
        
        skills = self.skills
        
        pyres_text = self.extract_resume()
        self.text = pyres_text
        
        ocr_ser = pd.Series(pyres_text)
        cleaned_words = hero.clean(ocr_ser) 
        
        main_df = pd.DataFrame(cleaned_words[0].split(), columns = ['text'])
        self.clean_data = main_df
        
        pri_score, pri_match = self.find_match(main_df, pd.DataFrame(skills['primary']))
        sec_score, sec_match = self.find_match(main_df, pd.DataFrame(skills['secondary']))

        doc_sim = self.resume_cosine_score(cleaned_words[0])
        
        return {'primary_score': pri_score,
                'primary_match': pri_match,
                'secondary_score': sec_score,
                'secondary_match': sec_match,
                'similarity': int(doc_sim)}
